#!/usr/bin/env python
import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner

if __name__ == "__main__":
    os.environ['DJANGO_SETTINGS_MODULE'] = 'whisperer.tests.test_settings'
    django.setup()

    verbosity_level = 2

    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=verbosity_level)
    failures = test_runner.run_tests(["whisperer.tests"])
    sys.exit(bool(failures))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2020-07-06 08:51
from __future__ import unicode_literals

from django.db import migrations

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('whisperer', '0004_webhook_retry_count'),
    ]

    operations = [
        migrations.AddField(
            model_name='webhook',
            name='config',
            field=JSONField(default=dict),
        ),
    ]



from distutils.version import StrictVersion

import django

if StrictVersion(django.get_version()) >= StrictVersion("2.0.0"):
    from django.utils.encoding import smart_str as smart
else:
    from django.utils.encoding import smart_text as smart

from whisperer import codes


class BaseException(Exception):
    code = codes.undefined

    def __init__(self, *args, **kwargs):
        if not isinstance(self.code, dict):
            raise Exception('parameter type must be a dict')
        code = self.code.get('code', 'undefined')
        message = getattr(self.codes, '%s' % code)
        self.message = message.get('en')
        self.obj = kwargs.get('obj', None)
        self.target = kwargs.get('target', None)
        self.params = kwargs.get('params')
        if self.params and isinstance(self.params, dict):
            self.message = smart(self.message).format(**self.params)
        elif self.params and isinstance(self.params, (list, set, tuple)):
            self.message = smart(self.message).format(*self.params)

        Exception.__init__(self, smart("{0}:{1}").format(code, self.message))

    def __new__(cls, *args, **kwargs):
        obj = super(BaseException, cls).__new__(cls)
        obj.__init__(*args, **kwargs)
        try:
            getattr(cls.codes, '%s' % obj.code.get('code'))
        except AttributeError:
            pass
        return obj

    @property
    def codes(self):
        return codes


class WebhookDoesNotExist(BaseException):
    code = codes.webhook_100_1


class WebhookAlreadyRegistered(BaseException):
    code = codes.webhook_100_2


class WebhookEventDoesNotExist(BaseException):
    code = codes.event_100_1


class EventAlreadyDelivered(BaseException):
    code = codes.event_100_3


class UnknownEventType(BaseException):
    code = codes.event_100_2


class ImproperlyConfigured(BaseException):
    code = codes.event_100_4


class AlreadyRegisteredEvent(BaseException):
    code = codes.event_100_5



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2023-12-14 09:09
from __future__ import unicode_literals

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ('whisperer', '0006_auto_20200925_1710'),
    ]

    operations = [
        migrations.CreateModel(
            name='EventQueue',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'created_date',
                    models.DateTimeField(
                        auto_now_add=True
                    )
                ),
                (
                    'modified_date',
                    models.DateTimeField(
                        auto_now=True,
                        db_index=True
                    )
                ),
                (
                    'uuid',
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        unique=True
                    )
                ),
                (
                    'hook_id',
                    models.PositiveIntegerField()
                ),
                (
                    'event_type',
                    models.CharField(
                        max_length=124
                    )
                ),
                (
                    'app_label',
                    models.CharField(
                        null=True,
                        blank=True,
                        max_length=124
                    )
                ),
                (
                    'model_name',
                    models.CharField(
                        null=True,
                        blank=True,
                        max_length=124
                    )
                ),
                (
                    'instance_pk',
                    models.PositiveIntegerField(
                        null=True
                    )
                ),
                (
                    'instance_dict',
                    JSONField(
                        default=dict
                    )
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]


import logging
from datetime import datetime

from celery import current_app
from django.core.cache import cache as django_cache
from django.utils.encoding import force_str
from django_redis import get_redis_connection

logger = logging.getLogger(__name__)


class LockTask(current_app.Task):
    """This abstract class ensures the same tasks run only once at a time"""

    abstract = True

    def __init__(self, *args, **kwargs):
        super(LockTask, self).__init__(*args, **kwargs)
        self.redis_cache = None

    def is_exists_cache_key(self, lock_cache_key):
        if self.redis_cache:
            return True if self.redis_cache.get(lock_cache_key) else False
        else:
            return True if django_cache.get(lock_cache_key) else False

    def get_redis_cache(self):
        if not self.redis_cache:
            try:
                self.redis_cache = get_redis_connection("default")
            except Exception as e:
                logger.info("Unable to connect to Redis. Error: %s" % str(e))
        return self.redis_cache

    def generate_lock_cache_key(self, *args, **kwargs):
        args_key = [force_str(arg) for arg in args]
        kwargs_key = [
            '{}_{}'.format(k, force_str(v)) for k, v in sorted(kwargs.items())
        ]
        return '_'.join([self.name] + args_key + kwargs_key)

    def __call__(self, *args, **kwargs):
        """Check task"""
        lock_cache_key = self.generate_lock_cache_key(*args, **kwargs)
        self.redis_cache = self.get_redis_cache()

        if not self.is_exists_cache_key(lock_cache_key):
            lock_time = datetime.now().isoformat()
            if self.redis_cache:
                self.redis_cache.set(
                    lock_cache_key, lock_time, nx=2 ** self.request.retries
                )
            else:
                django_cache.set(
                    lock_cache_key, lock_time, timeout=2 ** self.request.retries
                )
            return self.run(*args, **kwargs)
        else:
            logger.info("Task %s is already running.." % self.name)


class Registry(object):
    def __init__(self):
        self._registry = {}

    def __contains__(self, key):
        return key in self._registry

    def __getitem__(self, key):
        return self._registry[key]

    def register(self, key):
        def decorator(cls):
            self._registry[key] = cls
            return cls

        return decorator



# Generated by Django 3.0.4 on 2020-04-01 08:47

from django.db import migrations

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField


class Migration(migrations.Migration):
    dependencies = [
        ('whisperer', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='webhook',
            name='additional_headers',
            field=JSONField(default=dict),
        ),
    ]



from django.core.cache import cache as django_cache
from django.test import TestCase, override_settings
from django_redis import get_redis_connection
from mock import Mock, patch

from whisperer.utils import LockTask


class LockTaskTests(TestCase):

    redis_connection = get_redis_connection("default")

    class TestLockTask(LockTask):
        abstract = True
        name = 'test_lock_task'
        request = Mock(retries=3)

        def run(self, *args, **kwargs):
            # This is a test task that does nothing
            pass

    def tearDown(self):
        self.redis_connection.flushall()

    def test_lock_task_runs_with_redis_caching(self):
        # Test when Redis is available
        test_lock_task = self.TestLockTask()

        with patch.object(test_lock_task, 'run') as mock_run:
            test_lock_task.__call__("test_arg1", key2="test_arg2")
            mock_run.assert_called_once()

    def test_lock_task_not_run_with_redis_caching(self):
        # Test when Redis is available
        test_lock_task = self.TestLockTask()
        lock_cache_key = test_lock_task.generate_lock_cache_key(
            "test_arg1", key2="test_arg2"
        )
        self.redis_connection.set(lock_cache_key, "value")

        with patch.object(test_lock_task, 'run') as mock_run:
            test_lock_task.__call__("test_arg1", key2="test_arg2")
            mock_run.assert_not_called()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
            }
        }
    )
    def test_lock_task_runs_with_django_caching(self):
        # Test when Redis unavailable
        test_lock_task = self.TestLockTask()
        django_cache.clear()

        with patch.object(test_lock_task, 'run') as mock_run:
            test_lock_task.__call__("test_arg1", key2="test_arg2")
            mock_run.assert_called_once()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
            }
        }
    )
    def test_lock_task_not_run_with_django_caching(self):
        # Test when Redis unavailable
        test_lock_task = self.TestLockTask()
        lock_cache_key = test_lock_task.generate_lock_cache_key(
            "test_arg1", key2="test_arg2"
        )
        django_cache.set(lock_cache_key, "value")

        with patch.object(test_lock_task, 'run') as mock_run:
            test_lock_task.__call__("test_arg1", key2="test_arg2")
            mock_run.assert_not_called()
